#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Author:                     Hendrik Schutter, mail@hendrikschutter.com
"""

import cv2
import sys
import os
import subprocess

number_of_photos_in_scan = 3

def convert_to_avif(input_dir, output_dir):
    # print("input: " + str(input))
    # print("output: " + str(output))

    for path in os.listdir(input_dir):
        if os.path.isfile(os.path.join(input_dir, path)):
            input = os.path.join(input_dir, path)
            output = os.path.join(output_dir,(os.path.splitext(os.path.basename(os.path.normpath(input)))[0] + ".avif"))
            #print("Found extracted photo: " + str(input))
            #print("Convert to: " + str(output))

            cmd_handle = subprocess.Popen(["avifenc", "--jobs", "all", input, output], stdout=subprocess.PIPE)
            cmd_handle.communicate()[0]

            if cmd_handle.returncode != 0:
                raise Exception("Unable to convert via avifenc! Return code: " + str(cmd_handle.returncode))
                
            if os.path.exists(input):
                os.remove(input)

            cmd_handle = subprocess.Popen(["sync",],stdout=subprocess.PIPE,)
            cmd_handle.communicate()[0]

            if cmd_handle.returncode != 0:
                raise Exception("Unable to sync! Return code: " + str(cmd_handle.returncode))
    
def auto_crop_scan(src_path, output_dir):
    areas = list()
    areas.clear()

    # read the input image
    img_src = cv2.imread(src_path)

    if img_src is None:
        raise Exception("unable to read image " + str(input))

    # convert the image to grayscale
    img_gray = cv2.cvtColor(img_src, cv2.COLOR_BGR2GRAY)
    if img_gray is None:
        raise Exception("unable to convert image to grayscale")

    # apply thresholding on the gray image to create a binary image
    ret, thresh = cv2.threshold(img_gray, 127, 255, 0)

    # find the contours
    contours, hierarchy = cv2.findContours(
        thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE
    )

    for cnt in contours:
        areas.append(cv2.contourArea(cnt))

    areas.sort(reverse=True)

    if len(areas) >= (number_of_photos_in_scan+1):
        outer = areas[0]
        inter_min = areas[number_of_photos_in_scan]
        #print("Outer area: " + str(outer))
        #print("Inner area: " + str(inter_min))

        index = 0 # used for exported file name

        for cnt in contours:
            area = cv2.contourArea(cnt)
            if (area < outer) and (area >= inter_min):
                # compute the bounding rectangle of the contour
                x, y, w, h = cv2.boundingRect(cnt)

                x = x + 5  # TODO
                y = y + 5  # TODO
                w = w - 10  # TODO
                h = h - 10  # TODO

                img_crop = img_src[y : y + h, x : x + w]

                if img_crop is None:
                    raise Exception("unable to crop image")

                export_file_name = (
                    os.path.splitext(os.path.basename(os.path.normpath(src_path)))[0]
                    + "-"
                    + str(index)
                    + ".png"
                )

                export_file_path = os.path.join(output_dir, export_file_name)

                cv2.imwrite(export_file_path, img_crop)
                index = index + 1
    else:
        raise Exception("unable to find all photos in scan")

def main():
    print("starting ...")

    destination_dir_path = -1
    source_dir_path = -1
    tmp_dir_path = -1

    for argument in sys.argv:
        if argument.startswith("destination_dir"):
            destination_dir_path = argument.split("=")[1]
        if argument.startswith("source_dir"):
            source_dir_path = argument.split("=")[1]
        if argument.startswith("tmp_dir"):
            tmp_dir_path = argument.split("=")[1]

    if (destination_dir_path == -1) or (source_dir_path == -1) or (tmp_dir_path == -1):
        print("Unable to parse config!")
        print("Example usage:")
        print(
            "     python extract_photos.py source_dir=scans/ destination_dir=photos/ tmp_dir=tmp/"
        )
        sys.exit(-1)

    failed_extractions = 0

    for path in os.listdir(source_dir_path):
        if os.path.isfile(os.path.join(source_dir_path, path)):
            print("Found scan: " + str(os.path.join(source_dir_path, path)), end="")
            try:
                auto_crop_scan(os.path.join(source_dir_path, path), tmp_dir_path)
                convert_to_avif(tmp_dir_path, destination_dir_path)
                print("  →  Successfully extracted photos from scan")
            except Exception as e:
                print(str(e))
                failed_extractions = failed_extractions + 1

    if(failed_extractions == 0):
        print("Exiting without error")
        sys.exit(0)
    else:
        print("Unable to extract " + str(failed_extractions) + " scans.")
        print("Exiting with error")
        sys.exit(-1)

if __name__ == "__main__":
    main()
